@@ -1639,10 +1639,12 @@ drgn_program_read_memory(struct drgn_program *prog, void *buf, uint64_t address,
 {
 	uint64_t address_mask;
 	struct drgn_error *err = drgn_program_address_mask(prog, &address_mask);
+	if (err)
+		return err;
+	err = drgn_program_untagged_addr(prog, &address);
 	if (err)
 		return err;
 	char *p = buf;
-	address &= address_mask;
 	while (count > 0) {
 		size_t n = min((uint64_t)(count - 1), address_mask - address) + 1;
 		err = drgn_memory_reader_read(&prog->reader, p, address, n,
@@ -1662,13 +1664,11 @@ LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_read_c_string(struct drgn_program *prog, uint64_t address,
 			   bool physical, size_t max_size, char **ret)
 {
-	uint64_t address_mask;
-	struct drgn_error *err = drgn_program_address_mask(prog, &address_mask);
-	if (err)
-		return err;
 	_cleanup_(char_vector_deinit) struct char_vector str = VECTOR_INIT;
 	for (;;) {
-		address &= address_mask;
+		struct drgn_error *err = drgn_program_untagged_addr(prog, &address);
+		if (err)
+			return err;
 		char *c = char_vector_append_entry(&str);
 		if (!c)
 			return &drgn_enomem;
