@@ -26,11 +26,16 @@
 #include <linux/mm.h>
 #include <linux/module.h>
 #include <linux/netdevice.h>
+#include <linux/plist.h>
 #include <linux/radix-tree.h>
 #include <linux/rbtree.h>
 #include <linux/rbtree_augmented.h>
 #include <linux/skbuff.h>
 #include <linux/slab.h>
+#include <linux/stacktrace.h>
+#ifdef CONFIG_STACKDEPOT
+#include <linux/stackdepot.h>
+#endif
 #include <linux/vmalloc.h>
 #include <linux/wait.h>
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
@@ -151,6 +156,65 @@ static void drgn_test_llist_init(void)
 	llist_add(&drgn_test_singular_llist_entry.node, &drgn_test_singular_llist);
 }
 
+// plist
+
+PLIST_HEAD(drgn_test_empty_plist);
+PLIST_HEAD(drgn_test_full_plist);
+struct plist_node drgn_test_empty_plist_node =
+	PLIST_NODE_INIT(drgn_test_empty_plist_node, 50);
+
+struct drgn_test_plist_entry {
+	struct plist_node node;
+	char c;
+};
+
+struct drgn_test_plist_entry drgn_test_plist_entries[3];
+
+// Copy of plist_add() (minus debugging code) since it's not exported.
+static void drgn_plist_add(struct plist_node *node, struct plist_head *head)
+{
+	struct plist_node *first, *iter, *prev = NULL;
+	struct list_head *node_next = &head->node_list;
+
+	WARN_ON(!plist_node_empty(node));
+	WARN_ON(!list_empty(&node->prio_list));
+
+	if (plist_head_empty(head))
+		goto ins_node;
+
+	first = iter = plist_first(head);
+
+	do {
+		if (node->prio < iter->prio) {
+			node_next = &iter->node_list;
+			break;
+		}
+
+		prev = iter;
+		iter = list_entry(iter->prio_list.next,
+				struct plist_node, prio_list);
+	} while (iter != first);
+
+	if (!prev || prev->prio != node->prio)
+		list_add_tail(&node->prio_list, &iter->prio_list);
+ins_node:
+	list_add_tail(&node->node_list, node_next);
+}
+
+static void drgn_test_plist_init(void)
+{
+	plist_node_init(&drgn_test_plist_entries[0].node, 10);
+	drgn_test_plist_entries[0].c = 'H';
+	plist_node_init(&drgn_test_plist_entries[1].node, 20);
+	drgn_test_plist_entries[1].c = 'I';
+	plist_node_init(&drgn_test_plist_entries[2].node, 30);
+	drgn_test_plist_entries[2].c = '!';
+
+	drgn_plist_add(&drgn_test_plist_entries[1].node, &drgn_test_full_plist);
+	drgn_plist_add(&drgn_test_plist_entries[0].node, &drgn_test_full_plist);
+	drgn_plist_add(&drgn_test_plist_entries[2].node, &drgn_test_full_plist);
+}
+
 // mapletree
 
 const int drgn_test_have_maple_tree = HAVE_MAPLE_TREE;
@@ -650,6 +714,34 @@ static int drgn_test_slab_init(void)
 // kthread for stack trace
 
 static struct task_struct *drgn_test_kthread;
+
+const int drgn_test_have_stacktrace = IS_ENABLED(CONFIG_STACKTRACE);
+#ifdef CONFIG_STACKTRACE
+unsigned long drgn_test_stack_entries[16];
+unsigned int drgn_test_num_stack_entries;
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 2, 0)
+// Wrapper providing the newer interface and working around the caller of
+// save_stack_trace() not being included in the returned trace.
+static noinline unsigned int
+stack_trace_save(unsigned long *store, unsigned int size, unsigned int skipnr)
+{
+	struct stack_trace trace = {
+		.entries = store,
+		.max_entries = size,
+		.skip = skipnr,
+	};
+	save_stack_trace(&trace);
+	return trace.nr_entries;
+}
+#endif
+#endif
+
+const int drgn_test_have_stackdepot = IS_ENABLED(CONFIG_STACKDEPOT);
+#ifdef CONFIG_STACKDEPOT
+depot_stack_handle_t drgn_test_stack_handle;
+#endif
+
 // Completion indicating that the kthread has set up its stack frames and is
 // ready to be parked.
 static DECLARE_COMPLETION(drgn_test_kthread_ready);
@@ -777,6 +869,18 @@ static void drgn_test_kthread_fn3(void)
 	b = 2;
 	c = 3;
 
+#ifdef CONFIG_STACKTRACE
+	drgn_test_num_stack_entries = stack_trace_save(drgn_test_stack_entries,
+						       ARRAY_SIZE(drgn_test_stack_entries),
+						       0);
+#endif
+#ifdef CONFIG_STACKDEPOT
+	stack_depot_init();
+	drgn_test_stack_handle = stack_depot_save(drgn_test_stack_entries,
+						  drgn_test_num_stack_entries,
+						  GFP_KERNEL);
+#endif
+
 	complete(&drgn_test_kthread_ready);
 	for (;;) {
 		set_current_state(TASK_INTERRUPTIBLE);
@@ -1096,6 +1200,7 @@ static int __init drgn_test_init(void)
 
 	drgn_test_list_init();
 	drgn_test_llist_init();
+	drgn_test_plist_init();
 	ret = drgn_test_maple_tree_init();
 	if (ret)
 		goto out;
